from django import template
from django.test import TestCase
from django.template.loader import render_to_string

from django_navtag.templatetags.navtag import NavNode

BASIC_TEMPLATE = '''
{% load navtag %}
{% nav "banana" %}
{% if nav.apple %}Apple{% endif %}
{% if nav.banana %}Banana{% endif %}
'''

FOR_TEMPLATE = '''
{% load navtag %}
{% nav "banana" for othernav %}
{% if othernav.apple %}Apple{% endif %}
{% if othernav.banana %}Banana{% endif %}
'''


class NavTagTest(TestCase):

    def test_basic(self):
        t = template.Template(BASIC_TEMPLATE)
        content = t.render(template.Context()).strip()
        self.assertNotIn('Apple', content)
        self.assertIn('Banana', content)

    def test_for(self):
        t = template.Template(FOR_TEMPLATE)
        content = t.render(template.Context()).strip()
        self.assertNotIn('Apple', content)
        self.assertIn('Banana', content)

    def test_basic_extends(self):
        content = render_to_string('navtag_tests/home.txt').strip()
        self.assertIn('- Home (active)', content)
        self.assertNotIn('- Contact (active)', content)

        content = render_to_string('navtag_tests/contact.txt').strip()
        self.assertNotIn('- Home (active)', content)
        self.assertIn('- Contact (active)', content)

    def test_unset(self):
        content = render_to_string('navtag_tests/home.txt').strip()
        self.assertIn('- Home (active)', content)
        self.assertNotIn('- Contact (active)', content)

        content = render_to_string('navtag_tests/home-unset.txt').strip()
        self.assertNotIn('- Home (active)', content)
        self.assertNotIn('- Contact (active)', content)

    def test_heirarchical(self):
        content = render_to_string('navtag_tests/submenu/home.txt').strip()
        self.assertIn('- Home (active)', content)
        self.assertNotIn('- Fruit (active)', content)
        self.assertNotIn('  - Apple (active)', content)
        self.assertNotIn('  - Banana (active)', content)

        content = render_to_string(
            'navtag_tests/submenu/base_fruit.txt').strip()
        self.assertNotIn('- Home (active)', content)
        self.assertIn('- Fruit (active)', content)
        self.assertNotIn('  - Apple (active)', content)
        self.assertNotIn('  - Banana (active)', content)

        content = render_to_string('navtag_tests/submenu/apple.txt').strip()
        self.assertNotIn('- Home (active)', content)
        self.assertIn('- Fruit (active)', content)
        self.assertIn('  - Apple (active)', content)
        self.assertNotIn('  - Banana (active)', content)

        content = render_to_string('navtag_tests/submenu/banana.txt').strip()
        self.assertNotIn('- Home (active)', content)
        self.assertIn('- Fruit (active)', content)
        self.assertNotIn('  - Apple (active)', content)
        self.assertIn('  - Banana (active)', content)

    def test_top_context(self):
        content = render_to_string('navtag_tests/context/home.txt').strip()
        self.assertIn('- Home (active)', content)
        self.assertIn('HOME', content)

    def test_repr(self):
        node = NavNode()
        self.assertEqual(repr(node), "<Nav node>")

    def test_invalid_args(self):
        self.assertRaises(
            template.TemplateSyntaxError, template.Template,
            '''{% load navtag %}{% nav 'test' unexpected %}''')

    def test_backwards_compatible_empty_tag(self):
        content = template.Template(
            '{% load navtag %}{% nav %}').render(template.Context())
        self.assertEqual(content, '')

        content = template.Template(
            '{% load navtag %}{% nav for sidenav %}').render(template.Context())
        self.assertEqual(content, '')

    def test_yell_if_context_variable_changed(self):
        t = template.Template('{% load navtag %}{% nav "test" %}{{ nav }}')
        c = template.Context({'nav': 'anything'})
        c.update({'nav': 'test'})
        self.assertRaises(
            template.TemplateSyntaxError,
            t.render, c)



from django import template
from django.utils.encoding import smart_str


register = template.Library()


class NavNode(template.Node):

    def __init__(self, item=None, var_name=None):
        self.item = item
        self.var_name = var_name or 'nav'

    def render(self, context):
        # If self.item was blank, do nothing (it's only valid for backwards
        # compatibility).
        if not self.item:
            return ''

        first_context_stack = context.dicts[0]
        nav = first_context_stack.get(self.var_name)
        if nav is not context.get(self.var_name):
            raise template.TemplateSyntaxError(
                "'{}' variable has been altered in current context"
                .format(self.var_name))

        if nav:
            # If the nav variable is already set, don't do anything.
            return ''

        if not isinstance(nav, dict):
            nav = {}
            # Copy the stack to avoid leaking into other contexts.
            new_first_context_stack = first_context_stack.copy()
            new_first_context_stack[self.var_name] = nav
            context.dicts[0] = new_first_context_stack

        item = self.item.resolve(context)
        item = item and smart_str(item)
        value = True
        if not item:
            item = ''
        for part in reversed(item.split('.')):
            new_item = {}
            new_item[part] = value
            value = new_item

        nav.clear()
        nav.update(new_item)
        return ''

    def __repr__(self):
        return "<Nav node>"


@register.tag
def nav(parser, token):
    """
    Handles navigation item selection.

    Example usage::

        {# Set the context so {{ nav.home }} (or {{ mynav.home }}) is True #}
        {% nav "home" %} or {% nav "home" for mynav %}

    The most basic (and common) use of the tag is to call ``{% nav [item] %}``,
    where ``[item]`` is the item you want to check is selected.

    By default, this tag creates a ``nav`` context variable. To use an
    alternate context variable name, call ``{% nav [item] for [var_name] %}``.

    Your HTML navigation template should look something like::

        {% block nav %}
        <ul class="nav">
            <li{% if nav.home %} class="selected"{% endif %}>
                <a href="/">Home</a>
            </li>
            <li{% if nav.about %} class="selected"{% endif %}>
                <a href="/about/">About</a>
            </li>
        </ul>
        {% endblock %}

    To override this in a child template, you'd do::

        {% include "base.html" %}
        {% load nav %}

        {% block nav %}
        {% nav "about" %}
        {{ block.super }}
        {% endblock %}

    This works for multiple levels of template inheritance, due to the fact
    that the tag only does anything if the ``nav`` context variable does not
    exist. So only the first ``{% nav %}`` call found will ever be processed.

    To create a sub-menu you can check against, simply dot-separate the item::

        {% nav "about_menu.info" %}

    This will be pass for both ``{% if nav.about_menu %}`` and
    ``{% if nav.about_menu.info %}``.
    """
    bits = token.split_contents()
    if len(bits) > 2:
        var_name = bits.pop()
        for_bit = bits.pop()
        if for_bit != 'for' or len(bits) > 2:
            raise template.TemplateSyntaxError('Unexpected format for %s tag' %
                                               bits[0])
    else:
        var_name = 'nav'
    if len(bits) > 1:
        item = parser.compile_filter(bits[1])
    else:
        item = None
    return NavNode(item, var_name)



DATABASE_ENGINE = 'sqlite3'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'django_navtag',
]

SECRET_KEY = 'testing'



import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


class Tox(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import tox
        errno = tox.cmdline(self.test_args)
        sys.exit(errno)


setup(
    name='django-navtag',
    version='1.1',
    description=("A simple Django navigation template tag"),
    author='Chris Beaven',
    author_email='smileychris@gmail.com',
    url='http://github.com/SmileyChris/django-navtag',
    packages=find_packages(),
    include_package_data=True,
    tests_require=['tox'],
    cmdclass={'test': Tox},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ]
)
